class AsyncSwitchACLs:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getNetworkSwitchAccessControlLists(self, networkId: str):
        """
        **Return the access control lists for a MS network**
        https://api.meraki.com/api_docs#return-the-access-control-lists-for-a-ms-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['Switch ACLs'],
            'operation': 'getNetworkSwitchAccessControlLists',
        }
        resource = f'/networks/{networkId}/switch/accessControlLists'

        return await self._session.get(metadata, resource)

    async def updateNetworkSwitchAccessControlLists(self, networkId: str, rules: list):
        """
        **Update the access control lists for a MS network**
        https://api.meraki.com/api_docs#update-the-access-control-lists-for-a-ms-network
        
        - networkId (string)
        - rules (array): An ordered array of the access control list rules (not including the default rule). An empty array will clear the rules.
        """

        kwargs = locals()

        metadata = {
            'tags': ['Switch ACLs'],
            'operation': 'updateNetworkSwitchAccessControlLists',
        }
        resource = f'/networks/{networkId}/switch/accessControlLists'

        body_params = ['rules']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)




class AsyncSSIDs:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getNetworkDeviceWirelessStatus(self, networkId: str, serial: str):
        """
        **Return the SSID statuses of an access point**
        https://api.meraki.com/api_docs#return-the-ssid-statuses-of-an-access-point
        
        - networkId (string)
        - serial (string)
        """

        metadata = {
            'tags': ['SSIDs'],
            'operation': 'getNetworkDeviceWirelessStatus',
        }
        resource = f'/networks/{networkId}/devices/{serial}/wireless/status'

        return await self._session.get(metadata, resource)

    async def getNetworkSsids(self, networkId: str):
        """
        **List the SSIDs in a network. Supports networks with access points or wireless-enabled security appliances and teleworker gateways.**
        https://api.meraki.com/api_docs#list-the-ssids-in-a-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['SSIDs'],
            'operation': 'getNetworkSsids',
        }
        resource = f'/networks/{networkId}/ssids'

        return await self._session.get(metadata, resource)

    async def getNetworkSsid(self, networkId: str, number: str):
        """
        **Return a single SSID**
        https://api.meraki.com/api_docs#return-a-single-ssid
        
        - networkId (string)
        - number (string)
        """

        metadata = {
            'tags': ['SSIDs'],
            'operation': 'getNetworkSsid',
        }
        resource = f'/networks/{networkId}/ssids/{number}'

        return await self._session.get(metadata, resource)

    async def updateNetworkSsid(self, networkId: str, number: str, **kwargs):
        """
        **Update the attributes of an SSID**
        https://api.meraki.com/api_docs#update-the-attributes-of-an-ssid
        
        - networkId (string)
        - number (string)
        - name (string): The name of the SSID
        - enabled (boolean): Whether or not the SSID is enabled
        - authMode (string): The association control method for the SSID ('open', 'psk', 'open-with-radius', '8021x-meraki' or '8021x-radius')
        - enterpriseAdminAccess (string): Whether or not an SSID is accessible by 'enterprise' administrators ('access disabled' or 'access enabled')
        - encryptionMode (string): The psk encryption mode for the SSID ('wep' or 'wpa'). This param is only valid if the authMode is 'psk'
        - psk (string): The passkey for the SSID. This param is only valid if the authMode is 'psk'
        - wpaEncryptionMode (string): The types of WPA encryption. ('WPA1 only', 'WPA1 and WPA2', 'WPA2 only', 'WPA3 Transition Mode' or 'WPA3 only')
        - splashPage (string): The type of splash page for the SSID ('None', 'Click-through splash page', 'Billing', 'Password-protected with Meraki RADIUS', 'Password-protected with custom RADIUS', 'Password-protected with Active Directory', 'Password-protected with LDAP', 'SMS authentication', 'Systems Manager Sentry', 'Facebook Wi-Fi', 'Google OAuth' or 'Sponsored guest'). This attribute is not supported for template children.
        - radiusServers (array): The RADIUS 802.1x servers to be used for authentication. This param is only valid if the authMode is 'open-with-radius' or '8021x-radius'
        - radiusCoaEnabled (boolean): If true, Meraki devices will act as a RADIUS Dynamic Authorization Server and will respond to RADIUS Change-of-Authorization and Disconnect messages sent by the RADIUS server.
        - radiusFailoverPolicy (string): This policy determines how authentication requests should be handled in the event that all of the configured RADIUS servers are unreachable ('Deny access' or 'Allow access')
        - radiusLoadBalancingPolicy (string): This policy determines which RADIUS server will be contacted first in an authentication attempt and the ordering of any necessary retry attempts ('Strict priority order' or 'Round robin')
        - radiusAccountingEnabled (boolean): Whether or not RADIUS accounting is enabled. This param is only valid if the authMode is 'open-with-radius' or '8021x-radius'
        - radiusAccountingServers (array): The RADIUS accounting 802.1x servers to be used for authentication. This param is only valid if the authMode is 'open-with-radius' or '8021x-radius' and radiusAccountingEnabled is 'true'
        - radiusAttributeForGroupPolicies (string): Specify the RADIUS attribute used to look up group policies ('Filter-Id', 'Reply-Message', 'Airespace-ACL-Name' or 'Aruba-User-Role'). Access points must receive this attribute in the RADIUS Access-Accept message
        - ipAssignmentMode (string): The client IP assignment mode ('NAT mode', 'Bridge mode', 'Layer 3 roaming', 'Layer 3 roaming with a concentrator' or 'VPN')
        - useVlanTagging (boolean): Whether or not traffic should be directed to use specific VLANs. This param is only valid if the ipAssignmentMode is 'Bridge mode' or 'Layer 3 roaming'
        - concentratorNetworkId (string): The concentrator to use when the ipAssignmentMode is 'Layer 3 roaming with a concentrator' or 'VPN'.
        - vlanId (integer): The VLAN ID used for VLAN tagging. This param is only valid when the ipAssignmentMode is 'Layer 3 roaming with a concentrator' or 'VPN'
        - defaultVlanId (integer): The default VLAN ID used for 'all other APs'. This param is only valid when the ipAssignmentMode is 'Bridge mode' or 'Layer 3 roaming'
        - apTagsAndVlanIds (array): The list of tags and VLAN IDs used for VLAN tagging. This param is only valid when the ipAssignmentMode is 'Bridge mode' or 'Layer 3 roaming'
        - walledGardenEnabled (boolean): Allow access to a configurable list of IP ranges, which users may access prior to sign-on.
        - walledGardenRanges (string): Specify your walled garden by entering space-separated addresses, ranges using CIDR notation, domain names, and domain wildcards (e.g. 192.168.1.1/24 192.168.37.10/32 www.yahoo.com *.google.com). Meraki's splash page is automatically included in your walled garden.
        - radiusOverride (boolean): If true, the RADIUS response can override VLAN tag. This is not valid when ipAssignmentMode is 'NAT mode'.
        - minBitrate (number): The minimum bitrate in Mbps. ('1', '2', '5.5', '6', '9', '11', '12', '18', '24', '36', '48' or '54')
        - bandSelection (string): The client-serving radio frequencies. ('Dual band operation', '5 GHz band only' or 'Dual band operation with Band Steering')
        - perClientBandwidthLimitUp (integer): The upload bandwidth limit in Kbps. (0 represents no limit.)
        - perClientBandwidthLimitDown (integer): The download bandwidth limit in Kbps. (0 represents no limit.)
        - lanIsolationEnabled (boolean): Boolean indicating whether Layer 2 LAN isolation should be enabled or disabled. Only configurable when ipAssignmentMode is 'Bridge mode'.
        """

        kwargs.update(locals())

        if 'authMode' in kwargs:
            options = ['open', 'psk', 'open-with-radius', '8021x-meraki', '8021x-radius']
            assert kwargs['authMode'] in options, f'''"authMode" cannot be "{kwargs['authMode']}", & must be set to one of: {options}'''
        if 'enterpriseAdminAccess' in kwargs:
            options = ['access disabled', 'access enabled']
            assert kwargs['enterpriseAdminAccess'] in options, f'''"enterpriseAdminAccess" cannot be "{kwargs['enterpriseAdminAccess']}", & must be set to one of: {options}'''
        if 'encryptionMode' in kwargs:
            options = ['wep', 'wpa']
            assert kwargs['encryptionMode'] in options, f'''"encryptionMode" cannot be "{kwargs['encryptionMode']}", & must be set to one of: {options}'''
        if 'wpaEncryptionMode' in kwargs:
            options = ['WPA1 only', 'WPA1 and WPA2', 'WPA2 only', 'WPA3 Transition Mode', 'WPA3 only']
            assert kwargs['wpaEncryptionMode'] in options, f'''"wpaEncryptionMode" cannot be "{kwargs['wpaEncryptionMode']}", & must be set to one of: {options}'''
        if 'splashPage' in kwargs:
            options = ['None', 'Click-through splash page', 'Billing', 'Password-protected with Meraki RADIUS', 'Password-protected with custom RADIUS', 'Password-protected with Active Directory', 'Password-protected with LDAP', 'SMS authentication', 'Systems Manager Sentry', 'Facebook Wi-Fi', 'Google OAuth', 'Sponsored guest']
            assert kwargs['splashPage'] in options, f'''"splashPage" cannot be "{kwargs['splashPage']}", & must be set to one of: {options}'''
        if 'radiusFailoverPolicy' in kwargs:
            options = ['Deny access', 'Allow access']
            assert kwargs['radiusFailoverPolicy'] in options, f'''"radiusFailoverPolicy" cannot be "{kwargs['radiusFailoverPolicy']}", & must be set to one of: {options}'''
        if 'radiusLoadBalancingPolicy' in kwargs:
            options = ['Strict priority order', 'Round robin']
            assert kwargs['radiusLoadBalancingPolicy'] in options, f'''"radiusLoadBalancingPolicy" cannot be "{kwargs['radiusLoadBalancingPolicy']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['SSIDs'],
            'operation': 'updateNetworkSsid',
        }
        resource = f'/networks/{networkId}/ssids/{number}'

        body_params = ['name', 'enabled', 'authMode', 'enterpriseAdminAccess', 'encryptionMode', 'psk', 'wpaEncryptionMode', 'splashPage', 'radiusServers', 'radiusCoaEnabled', 'radiusFailoverPolicy', 'radiusLoadBalancingPolicy', 'radiusAccountingEnabled', 'radiusAccountingServers', 'radiusAttributeForGroupPolicies', 'ipAssignmentMode', 'useVlanTagging', 'concentratorNetworkId', 'vlanId', 'defaultVlanId', 'apTagsAndVlanIds', 'walledGardenEnabled', 'walledGardenRanges', 'radiusOverride', 'minBitrate', 'bandSelection', 'perClientBandwidthLimitUp', 'perClientBandwidthLimitDown', 'lanIsolationEnabled']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)




import json
import time

import ssl
import aiohttp
import asyncio

from meraki.config import *
from meraki.exceptions import *


# Main module interface
class AsyncRestSession:
    def __init__(
        self,
        logger,
        api_key,
        base_url=DEFAULT_BASE_URL,
        single_request_timeout=SINGLE_REQUEST_TIMEOUT,
        certificate_path=CERTIFICATE_PATH,
        wait_on_rate_limit=WAIT_ON_RATE_LIMIT,
        maximum_retries=MAXIMUM_RETRIES,
        simulate=SIMULATE_API_CALLS,
    ):
        super().__init__()

        # Initialize attributes and properties
        self._api_key = str(api_key)
        self._base_url = str(base_url)
        self._single_request_timeout = single_request_timeout
        self._certificate_path = certificate_path
        self._wait_on_rate_limit = wait_on_rate_limit
        self._maximum_retries = maximum_retries
        self._simulate = simulate

        # Update the headers of the `requests` session
        headers = None
        if "v0" in self._base_url:
            headers = {
                "X-Cisco-Meraki-API-Key": self._api_key,
                "Content-Type": "application/json",
            }
        elif "v1" in self._base_url:
            headers = {
                "Authorization": "Bearer " + self._api_key,
                "Content-Type": "application/json",
            }
        if self._certificate_path:
            self._sslcontext = ssl.create_default_context()
            self._sslcontext.load_verify_locations(certificate_path)

        # Initialize a new `aiohttp` session
        self._req_session = aiohttp.ClientSession(
            headers=headers,
            timeout=aiohttp.ClientTimeout(total=single_request_timeout),
        )

        # Log API calls
        self._logger = logger
        self._parameters = locals()
        self._parameters["api_key"] = "*" * 36 + self._api_key[-4:]
        self._logger.info(
            f"Meraki dashboard API session initialized with these parameters: {self._parameters}"
        )

    async def request(self, metadata, method, url, **kwargs):
        # Metadata on endpoint
        tag = metadata["tags"][0]
        operation = metadata["operation"]

        # Update request kwargs with session defaults
        if self._certificate_path:
            kwargs.setdefault("ssl", self._sslcontext)
        kwargs.setdefault("timeout", self._single_request_timeout)

        # Ensure proper base URL
        if "meraki.com" in url:
            abs_url = url
        else:
            abs_url = self._base_url + url

        # Set maximum number of retries
        retries = self._maximum_retries

        # Option to simulate non-safe API calls without actually sending them
        self._logger.debug(metadata)
        if self._simulate and method != "GET":
            self._logger.info(f"{tag}, {operation} - SIMULATED")
            return None
        else:
            response = None
            for _ in range(retries):
                # Make the HTTP request to the API endpoint
                try:
                    response = await self._req_session.request(
                        method, abs_url, **kwargs
                    )
                    reason = response.reason if response.reason else None
                    status = response.status
                except Exception as e:
                    self._logger.warning(
                        f"{tag}, {operation} - {e}, retrying in 1 second"
                    )
                    await asyncio.sleep(1)
                    continue

                if status == 200:
                    if "page" in metadata:
                        counter = metadata["page"]
                        self._logger.info(
                            f"{tag}, {operation}; page {counter} - {status} {reason}"
                        )
                    else:
                        self._logger.info(f"{tag}, {operation} - {status} {reason}")
                    # For non-empty response to GET, ensure valid JSON
                    try:
                        if method == "GET":
                            await response.json()
                        return response
                    except (
                        json.decoder.JSONDecodeError,
                        aiohttp.client_exceptions.ContentTypeError,
                    ) as e:
                        self._logger.warning(
                            f"{tag}, {operation} - {e}, retrying in 1 second"
                        )
                        await asyncio.sleep(1)
                # Handle 3XX redirects automatically
                elif 300 <= status < 400:
                    abs_url = response.headers["Location"]
                    substring = "meraki.com/api/v"
                    self._base_url = abs_url[
                        : abs_url.find(substring) + len(substring) + 1
                    ]
                # Rate limit 429 errors
                elif status == 429:
                    wait = int(response.headers["Retry-After"])
                    self._logger.warning(
                        f"{tag}, {operation} - {status} {reason}, retrying in {wait} seconds"
                    )
                    await asyncio.sleep(wait)
                # 5XX errors
                elif status >= 500:
                    self._logger.warning(
                        f"{tag}, {operation} - {status} {reason}, retrying in 1 second"
                    )
                    await asyncio.sleep(1)
                # 4XX errors
                else:
                    try:
                        message = await response.json()
                    except aiohttp.client_exceptions.ContentTypeError:
                        message = (await response.text())[:100]
                    self._logger.error(
                        f"{tag}, {operation} - {status} {reason}, {message}"
                    )
            raise AsyncAPIError(metadata, response, await response.text())

    async def get(self, metadata, url, params=None):
        metadata["method"] = "GET"
        metadata["url"] = url
        metadata["params"] = params
        response = await self.request(metadata, "GET", url, params=params)
        return await response.json()

    async def get_pages(
        self, metadata, url, params=None, total_pages=-1, direction="next"
    ):
        if type(total_pages) == str and total_pages.lower() == "all":
            total_pages = -1
        metadata["page"] = 1

        response = await self.request(metadata, "GET", url, params=params)
        results = await response.json()

        # Get additional pages if more than one requested
        while total_pages != 1:
            # Parse Link from headers
            links = response.headers["Link"].split(",")
            first = prev = next = last = None
            for l in links:
                if "rel=first" in l:
                    first = l[l.find("<") + 1 : l.find(">")]
                elif "rel=prev" in l:
                    prev = l[l.find("<") + 1 : l.find(">")]
                elif "rel=next" in l:
                    next = l[l.find("<") + 1 : l.find(">")]
                elif "rel=last" in l:
                    last = l[l.find("<") + 1 : l.find(">")]

            # GET the subsequent page
            if direction == "next" and next:
                metadata["page"] += 1
                response = await self.request(metadata, "GET", next)
            elif direction == "prev" and prev:
                metadata["page"] += 1
                response = await self.request(metadata, "GET", prev)
            else:
                break

            # Append that page's results, depending on the endpoint
            if type(results) == list:
                results.extend(await response.json())
            # For event log endpoint
            elif type(results) == dict:
                json_response = await response.json()
                start = json_response["pageStartAt"]
                end = json_response["pageEndAt"]
                events = json_response["events"]
                if start < results["pageStartAt"]:
                    results["pageStartAt"] = start
                if end > results["pageEndAt"]:
                    results["pageEndAt"] = end
                results["events"].extend(events)

            total_pages -= 1

        return results

    async def post(self, metadata, url, json=None):
        metadata["method"] = "POST"
        metadata["url"] = url
        metadata["json"] = json
        response = await self.request(metadata, "POST", url, json=json)
        return await response.json()

    async def put(self, metadata, url, json=None):
        metadata["method"] = "PUT"
        metadata["url"] = url
        metadata["json"] = json
        response = await self.request(metadata, "PUT", url, json=json)
        return await response.json()

    async def delete(self, metadata, url):
        metadata["method"] = "DELETE"
        metadata["url"] = url
        await self.request(metadata, "DELETE", url)
        return None

    async def close(self):
        await self._req_session.close()



class IntrusionSettings(object):
    def __init__(self, session):
        super(IntrusionSettings, self).__init__()
        self._session = session
    
    def getNetworkSecurityIntrusionSettings(self, networkId: str):
        """
        **Returns all supported intrusion settings for an MX network**
        https://api.meraki.com/api_docs#returns-all-supported-intrusion-settings-for-an-mx-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['Intrusion settings'],
            'operation': 'getNetworkSecurityIntrusionSettings',
        }
        resource = f'/networks/{networkId}/security/intrusionSettings'

        return self._session.get(metadata, resource)

    def updateNetworkSecurityIntrusionSettings(self, networkId: str, **kwargs):
        """
        **Set the supported intrusion settings for an MX network**
        https://api.meraki.com/api_docs#set-the-supported-intrusion-settings-for-an-mx-network
        
        - networkId (string)
        - mode (string): Set mode to 'disabled'/'detection'/'prevention' (optional - omitting will leave current config unchanged)
        - idsRulesets (string): Set the detection ruleset 'connectivity'/'balanced'/'security' (optional - omitting will leave current config unchanged). Default value is 'balanced' if none currently saved
        - protectedNetworks (object): Set the included/excluded networks from the intrusion engine (optional - omitting will leave current config unchanged). This is available only in 'passthrough' mode
        """

        kwargs.update(locals())

        if 'mode' in kwargs:
            options = ['prevention', 'detection', 'disabled']
            assert kwargs['mode'] in options, f'''"mode" cannot be "{kwargs['mode']}", & must be set to one of: {options}'''
        if 'idsRulesets' in kwargs:
            options = ['connectivity', 'balanced', 'security']
            assert kwargs['idsRulesets'] in options, f'''"idsRulesets" cannot be "{kwargs['idsRulesets']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['Intrusion settings'],
            'operation': 'updateNetworkSecurityIntrusionSettings',
        }
        resource = f'/networks/{networkId}/security/intrusionSettings'

        body_params = ['mode', 'idsRulesets', 'protectedNetworks']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)

    def getOrganizationSecurityIntrusionSettings(self, organizationId: str):
        """
        **Returns all supported intrusion settings for an organization**
        https://api.meraki.com/api_docs#returns-all-supported-intrusion-settings-for-an-organization
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['Intrusion settings'],
            'operation': 'getOrganizationSecurityIntrusionSettings',
        }
        resource = f'/organizations/{organizationId}/security/intrusionSettings'

        return self._session.get(metadata, resource)

    def updateOrganizationSecurityIntrusionSettings(self, organizationId: str, whitelistedRules: list):
        """
        **Sets supported intrusion settings for an organization**
        https://api.meraki.com/api_docs#sets-supported-intrusion-settings-for-an-organization
        
        - organizationId (string)
        - whitelistedRules (array): Sets a list of specific SNORT® signatures to whitelist
        """

        kwargs = locals()

        metadata = {
            'tags': ['Intrusion settings'],
            'operation': 'updateOrganizationSecurityIntrusionSettings',
        }
        resource = f'/organizations/{organizationId}/security/intrusionSettings'

        body_params = ['whitelistedRules']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)




class MRL3Firewall(object):
    def __init__(self, session):
        super(MRL3Firewall, self).__init__()
        self._session = session
    
    def getNetworkSsidL3FirewallRules(self, networkId: str, number: str):
        """
        **Return the L3 firewall rules for an SSID on an MR network**
        https://api.meraki.com/api_docs#return-the-l3-firewall-rules-for-an-ssid-on-an-mr-network
        
        - networkId (string)
        - number (string)
        """

        metadata = {
            'tags': ['MR L3 firewall'],
            'operation': 'getNetworkSsidL3FirewallRules',
        }
        resource = f'/networks/{networkId}/ssids/{number}/l3FirewallRules'

        return self._session.get(metadata, resource)

    def updateNetworkSsidL3FirewallRules(self, networkId: str, number: str, **kwargs):
        """
        **Update the L3 firewall rules of an SSID on an MR network**
        https://api.meraki.com/api_docs#update-the-l3-firewall-rules-of-an-ssid-on-an-mr-network
        
        - networkId (string)
        - number (string)
        - rules (array): An ordered array of the firewall rules for this SSID (not including the local LAN access rule or the default rule)
        - allowLanAccess (boolean): Allow wireless client access to local LAN (boolean value - true allows access and false denies access) (optional)
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['MR L3 firewall'],
            'operation': 'updateNetworkSsidL3FirewallRules',
        }
        resource = f'/networks/{networkId}/ssids/{number}/l3FirewallRules'

        body_params = ['rules', 'allowLanAccess']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)




class SyslogServers(object):
    def __init__(self, session):
        super(SyslogServers, self).__init__()
        self._session = session
    
    def getNetworkSyslogServers(self, networkId: str):
        """
        **List the syslog servers for a network**
        https://api.meraki.com/api_docs#list-the-syslog-servers-for-a-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['Syslog servers'],
            'operation': 'getNetworkSyslogServers',
        }
        resource = f'/networks/{networkId}/syslogServers'

        return self._session.get(metadata, resource)

    def updateNetworkSyslogServers(self, networkId: str, servers: list):
        """
        **Update the syslog servers for a network**
        https://api.meraki.com/api_docs#update-the-syslog-servers-for-a-network
        
        - networkId (string)
        - servers (array): A list of the syslog servers for this network
        """

        kwargs = locals()

        metadata = {
            'tags': ['Syslog servers'],
            'operation': 'updateNetworkSyslogServers',
        }
        resource = f'/networks/{networkId}/syslogServers'

        body_params = ['servers']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)




class SwitchStacks(object):
    def __init__(self, session):
        super(SwitchStacks, self).__init__()
        self._session = session
    
    def getNetworkSwitchStacks(self, networkId: str):
        """
        **List the switch stacks in a network**
        https://api.meraki.com/api_docs#list-the-switch-stacks-in-a-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['Switch stacks'],
            'operation': 'getNetworkSwitchStacks',
        }
        resource = f'/networks/{networkId}/switchStacks'

        return self._session.get(metadata, resource)

    def createNetworkSwitchStack(self, networkId: str, name: str, serials: list):
        """
        **Create a stack**
        https://api.meraki.com/api_docs#create-a-stack
        
        - networkId (string)
        - name (string): The name of the new stack
        - serials (array): An array of switch serials to be added into the new stack
        """

        kwargs = locals()

        metadata = {
            'tags': ['Switch stacks'],
            'operation': 'createNetworkSwitchStack',
        }
        resource = f'/networks/{networkId}/switchStacks'

        body_params = ['name', 'serials']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.post(metadata, resource, payload)

    def getNetworkSwitchStack(self, networkId: str, switchStackId: str):
        """
        **Show a switch stack**
        https://api.meraki.com/api_docs#show-a-switch-stack
        
        - networkId (string)
        - switchStackId (string)
        """

        metadata = {
            'tags': ['Switch stacks'],
            'operation': 'getNetworkSwitchStack',
        }
        resource = f'/networks/{networkId}/switchStacks/{switchStackId}'

        return self._session.get(metadata, resource)

    def deleteNetworkSwitchStack(self, networkId: str, switchStackId: str):
        """
        **Delete a stack**
        https://api.meraki.com/api_docs#delete-a-stack
        
        - networkId (string)
        - switchStackId (string)
        """

        metadata = {
            'tags': ['Switch stacks'],
            'operation': 'deleteNetworkSwitchStack',
        }
        resource = f'/networks/{networkId}/switchStacks/{switchStackId}'

        return self._session.delete(metadata, resource)

    def addNetworkSwitchStack(self, networkId: str, switchStackId: str, serial: str):
        """
        **Add a switch to a stack**
        https://api.meraki.com/api_docs#add-a-switch-to-a-stack
        
        - networkId (string)
        - switchStackId (string)
        - serial (string): The serial of the switch to be added
        """

        kwargs = locals()

        metadata = {
            'tags': ['Switch stacks'],
            'operation': 'addNetworkSwitchStack',
        }
        resource = f'/networks/{networkId}/switchStacks/{switchStackId}/add'

        body_params = ['serial']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.post(metadata, resource, payload)

    def removeNetworkSwitchStack(self, networkId: str, switchStackId: str, serial: str):
        """
        **Remove a switch from a stack**
        https://api.meraki.com/api_docs#remove-a-switch-from-a-stack
        
        - networkId (string)
        - switchStackId (string)
        - serial (string): The serial of the switch to be removed
        """

        kwargs = locals()

        metadata = {
            'tags': ['Switch stacks'],
            'operation': 'removeNetworkSwitchStack',
        }
        resource = f'/networks/{networkId}/switchStacks/{switchStackId}/remove'

        body_params = ['serial']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.post(metadata, resource, payload)




import json
import time

import requests

from .config import *
from .exceptions import *


# Main module interface
class RestSession(object):
    def __init__(
        self,
        logger,
        api_key,
        base_url=DEFAULT_BASE_URL,
        single_request_timeout=SINGLE_REQUEST_TIMEOUT,
        certificate_path=CERTIFICATE_PATH,
        wait_on_rate_limit=WAIT_ON_RATE_LIMIT,
        maximum_retries=MAXIMUM_RETRIES,
        simulate=SIMULATE_API_CALLS,
    ):
        super(RestSession, self).__init__()

        # Initialize attributes and properties
        self._api_key = str(api_key)
        self._base_url = str(base_url)
        self._single_request_timeout = single_request_timeout
        self._certificate_path = certificate_path
        self._wait_on_rate_limit = wait_on_rate_limit
        self._maximum_retries = maximum_retries
        self._simulate = simulate

        # Initialize a new `requests` session
        self._req_session = requests.session()

        # Remove unneeded trailing slash in base URL if present
        if self._base_url[-1] == '/':
            self._base_url = self._base_url[:-1]

        # Update the headers of the `requests` session
        if 'v0' in self._base_url:
            self._req_session.headers = {'X-Cisco-Meraki-API-Key': self._api_key, 'Content-Type': 'application/json'}
        elif 'v1' in self._base_url:
            self._req_session.headers = {'Authorization': 'Bearer ' + self._api_key, 'Content-Type': 'application/json'}

        # Log API calls
        self._logger = logger
        self._parameters = locals()
        self._parameters['api_key'] = '*' * 36 + self._api_key[-4:]
        self._logger.info(f'Meraki dashboard API session initialized with these parameters: {self._parameters}')

    def request(self, metadata, method, url, **kwargs):
        # Metadata on endpoint
        tag = metadata['tags'][0]
        operation = metadata['operation']

        # Update request kwargs with session defaults
        if self._certificate_path:
            kwargs.setdefault('verify', self._certificate_path)
        kwargs.setdefault('timeout', self._single_request_timeout)

        # Ensure proper base URL
        if 'meraki.com' in url:
            abs_url = url
        else:
            abs_url = self._base_url + url

        # Set maximum number of retries
        retries = self._maximum_retries

        # Option to simulate non-safe API calls without actually sending them
        self._logger.debug(metadata)
        if self._simulate and method != 'GET':
            self._logger.info(f'{tag}, {operation} - SIMULATED')
            return None
        else:
            response = None
            while retries > 0:
                # Make the HTTP request to the API endpoint
                try:
                    response = self._req_session.request(method, abs_url, allow_redirects=False, **kwargs)
                    reason = response.reason if response.reason else ''
                    status = response.status_code
                except requests.exceptions.RequestException as e:
                    self._logger.warning(f'{tag}, {operation} - {e}, retrying in 1 second')
                    time.sleep(1)
                    retries -= 1
                    if retries == 0:
                        raise APIError(metadata, response)
                    else:
                        continue

                # Handle 3XX redirects automatically
                if str(status)[0] == '3':
                    abs_url = response.headers['Location']
                    substring = 'meraki.com/api/v'
                    self._base_url = abs_url[:abs_url.find(substring) + len(substring) + 1]

                # 2XX success
                elif response.ok:
                    if 'page' in metadata:
                        counter = metadata['page']
                        self._logger.info(f'{tag}, {operation}; page {counter} - {status} {reason}')
                    else:
                        self._logger.info(f'{tag}, {operation} - {status} {reason}')
                    # For non-empty response to GET, ensure valid JSON
                    try:
                        if method == 'GET' and response.text.strip():
                            response.json()
                        return response
                    except json.decoder.JSONDecodeError as e:
                        self._logger.warning(f'{tag}, {operation} - {e}, retrying in 1 second')
                        time.sleep(1)
                        retries -= 1
                        if retries == 0:
                            raise APIError(metadata, response)
                        else:
                            continue

                # Rate limit 429 errors
                elif status == 429:
                    wait = int(response.headers['Retry-After'])
                    self._logger.warning(f'{tag}, {operation} - {status} {reason}, retrying in {wait} seconds')
                    time.sleep(wait)
                    retries -= 1
                    if retries == 0:
                        raise APIError(metadata, response)

                # 5XX errors
                elif status >= 500:
                    self._logger.warning(f'{tag}, {operation} - {status} {reason}, retrying in 1 second')
                    time.sleep(1)
                    retries -= 1
                    if retries == 0:
                        raise APIError(metadata, response)

                # 4XX errors
                else:
                    try:
                        message = response.json()
                    except ValueError:
                        message = response.text[:100]

                    # Check specifically for action batch concurrency error
                    action_batch_concurrency_error = {
                        'errors': [
                            'Too many concurrently executing batches. Maximum is 5 confirmed but not yet executed batches.'
                        ]
                    }
                    if message == action_batch_concurrency_error:
                        self._logger.warning(f'{tag}, {operation} - {status} {reason}, retrying in 60 seconds')
                        time.sleep(60)
                        retries -= 1
                        if retries == 0:
                            raise APIError(metadata, response)

                    # All other client-side errors
                    else:
                        self._logger.error(f'{tag}, {operation} - {status} {reason}, {message}')
                        raise APIError(metadata, response)

    def get(self, metadata, url, params=None):
        metadata['method'] = 'GET'
        metadata['url'] = url
        metadata['params'] = params
        response = self.request(metadata, 'GET', url, params=params)
        return response.json() if response and response.text.strip() else None

    def get_pages(self, metadata, url, params=None, total_pages=-1, direction='next'):
        if type(total_pages) == str and total_pages.lower() == 'all':
            total_pages = -1
        elif type(total_pages) == str and total_pages.isnumeric():
            total_pages = int(total_pages)
        metadata['page'] = 1

        response = self.request(metadata, 'GET', url, params=params)
        results = response.json()

        # Get additional pages if more than one requested
        while total_pages != 1:
            # Parse Link from headers
            links = response.headers['Link'].split(',')
            first = prev = next = last = None
            for l in links:
                if 'rel=first' in l:
                    first = l[l.find('<')+1:l.find('>')]
                elif 'rel=prev' in l:
                    prev = l[l.find('<')+1:l.find('>')]
                elif 'rel=next' in l:
                    next = l[l.find('<')+1:l.find('>')]
                elif 'rel=last' in l:
                    last = l[l.find('<')+1:l.find('>')]

            # GET the subsequent page
            if direction == 'next' and next:
                metadata['page'] += 1
                response = self.request(metadata, 'GET', next)
            elif direction == 'prev' and prev:
                metadata['page'] += 1
                response = self.request(metadata, 'GET', prev)
            else:
                break

            # Append that page's results, depending on the endpoint
            if type(results) == list:
                results.extend(response.json())
            # For event log endpoint
            elif type(results) == dict:
                start = response.json()['pageStartAt']
                end = response.json()['pageEndAt']
                events = response.json()['events']
                if start < results['pageStartAt']:
                    results['pageStartAt'] = start
                if end > results['pageEndAt']:
                    results['pageEndAt'] = end
                results['events'].extend(events)

            total_pages -= 1

        return results

    def post(self, metadata, url, json=None):
        metadata['method'] = 'POST'
        metadata['url'] = url
        metadata['json'] = json
        response = self.request(metadata, 'POST', url, json=json)
        return response.json() if response and response.text.strip() else None

    def put(self, metadata, url, json=None):
        metadata['method'] = 'PUT'
        metadata['url'] = url
        metadata['json'] = json
        response = self.request(metadata, 'PUT', url, json=json)
        return response.json() if response and response.text.strip() else None

    def delete(self, metadata, url):
        metadata['method'] = 'DELETE'
        metadata['url'] = url
        self.request(metadata, 'DELETE', url)
        return None



class ManagementInterfaceSettings(object):
    def __init__(self, session):
        super(ManagementInterfaceSettings, self).__init__()
        self._session = session
    
    def getNetworkDeviceManagementInterfaceSettings(self, networkId: str, serial: str):
        """
        **Return the management interface settings for a device**
        https://api.meraki.com/api_docs#return-the-management-interface-settings-for-a-device
        
        - networkId (string)
        - serial (string)
        """

        metadata = {
            'tags': ['Management interface settings'],
            'operation': 'getNetworkDeviceManagementInterfaceSettings',
        }
        resource = f'/networks/{networkId}/devices/{serial}/managementInterfaceSettings'

        return self._session.get(metadata, resource)

    def updateNetworkDeviceManagementInterfaceSettings(self, networkId: str, serial: str, **kwargs):
        """
        **Update the management interface settings for a device**
        https://api.meraki.com/api_docs#update-the-management-interface-settings-for-a-device
        
        - networkId (string)
        - serial (string)
        - wan1 (object): WAN 1 settings
        - wan2 (object): WAN 2 settings (only for MX devices)
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['Management interface settings'],
            'operation': 'updateNetworkDeviceManagementInterfaceSettings',
        }
        resource = f'/networks/{networkId}/devices/{serial}/managementInterfaceSettings'

        body_params = ['wan1', 'wan2']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)




class AsyncContentFilteringRules:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getNetworkContentFiltering(self, networkId: str):
        """
        **Return the content filtering settings for an MX network**
        https://api.meraki.com/api_docs#return-the-content-filtering-settings-for-an-mx-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['Content filtering rules'],
            'operation': 'getNetworkContentFiltering',
        }
        resource = f'/networks/{networkId}/contentFiltering'

        return await self._session.get(metadata, resource)

    async def updateNetworkContentFiltering(self, networkId: str, **kwargs):
        """
        **Update the content filtering settings for an MX network**
        https://api.meraki.com/api_docs#update-the-content-filtering-settings-for-an-mx-network
        
        - networkId (string)
        - allowedUrlPatterns (array): A whitelist of URL patterns to allow
        - blockedUrlPatterns (array): A blacklist of URL patterns to block
        - blockedUrlCategories (array): A list of URL categories to block
        - urlCategoryListSize (string): URL category list size which is either 'topSites' or 'fullList'
        """

        kwargs.update(locals())

        if 'urlCategoryListSize' in kwargs:
            options = ['topSites', 'fullList']
            assert kwargs['urlCategoryListSize'] in options, f'''"urlCategoryListSize" cannot be "{kwargs['urlCategoryListSize']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['Content filtering rules'],
            'operation': 'updateNetworkContentFiltering',
        }
        resource = f'/networks/{networkId}/contentFiltering'

        body_params = ['allowedUrlPatterns', 'blockedUrlPatterns', 'blockedUrlCategories', 'urlCategoryListSize']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)




class MXL7ApplicationCategories(object):
    def __init__(self, session):
        super(MXL7ApplicationCategories, self).__init__()
        self._session = session
    
    def getNetworkL7FirewallRulesApplicationCategories(self, networkId: str):
        """
        **Return the L7 firewall application categories and their associated applications for an MX network**
        https://api.meraki.com/api_docs#return-the-l7-firewall-application-categories-and-their-associated-applications-for-an-mx-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['MX L7 application categories'],
            'operation': 'getNetworkL7FirewallRulesApplicationCategories',
        }
        resource = f'/networks/{networkId}/l7FirewallRules/applicationCategories'

        return self._session.get(metadata, resource)




class AsyncMXCellularFirewall:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getNetworkCellularFirewallRules(self, networkId: str):
        """
        **Return the cellular firewall rules for an MX network**
        https://api.meraki.com/api_docs#return-the-cellular-firewall-rules-for-an-mx-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['MX cellular firewall'],
            'operation': 'getNetworkCellularFirewallRules',
        }
        resource = f'/networks/{networkId}/cellularFirewallRules'

        return await self._session.get(metadata, resource)

    async def updateNetworkCellularFirewallRules(self, networkId: str, **kwargs):
        """
        **Update the cellular firewall rules of an MX network**
        https://api.meraki.com/api_docs#update-the-cellular-firewall-rules-of-an-mx-network
        
        - networkId (string)
        - rules (array): An ordered array of the firewall rules (not including the default rule)
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['MX cellular firewall'],
            'operation': 'updateNetworkCellularFirewallRules',
        }
        resource = f'/networks/{networkId}/cellularFirewallRules'

        body_params = ['rules']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)




class OpenAPISpec(object):
    def __init__(self, session):
        super(OpenAPISpec, self).__init__()
        self._session = session
    
    def getOrganizationOpenapiSpec(self, organizationId: str):
        """
        **Return the OpenAPI 2.0 Specification of the organization's API documentation in JSON**
        https://api.meraki.com/api_docs#return-the-openapi-2
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['OpenAPI Spec'],
            'operation': 'getOrganizationOpenapiSpec',
        }
        resource = f'/organizations/{organizationId}/openapiSpec'

        return self._session.get(metadata, resource)




'''
SPECIAL THANKS to Heimo Stieg (https://github.com/coreGreenberet) for implementing the "aio_" examples as well as this
script, which generates the contents of the "aio" directory for running asynchronously.
'''


import csv
from datetime import datetime
import os
import logging
import re
import io

_logger = logging.Logger("")


def create_logger(log_file_prefix, print_console) -> logging.Logger:
    logger = logging.getLogger(__name__)
    log_file = f"{log_file_prefix}_log__{datetime.now():%Y-%m-%d_%H-%M-%S}.log"
    logging.basicConfig(
        filename=log_file,
        level=logging.DEBUG,
        format="%(asctime)s %(name)12s: %(levelname)8s > %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
    )

    if print_console:
        console = logging.StreamHandler()
        console.setLevel(logging.DEBUG)
        formatter = logging.Formatter("%(name)12s: %(levelname)8s > %(message)s")
        console.setFormatter(formatter)
        logging.getLogger("").addHandler(console)

    return logger


def get_aio_path(path, filename) -> str:
    """helper function to return the correct for a converted file based on the original file path"""

    if not os.path.isfile(os.path.join(path, filename)):
        raise ValueError(f"{os.path.join(path,filename)} is not a file")

    if "api" in path:
        path, _ = os.path.split(path)
        return os.path.join(path, "aio", "api", filename)
    else:
        return os.path.join(path, "aio", filename)


def read_file(filepath):
    """reads the file into memory and returns it contents"""

    _logger.info(f"Reading {filepath}")
    # read file into memory
    contents = None
    with open(filepath, "r") as f:
        contents = f.read()
    return contents


def compile_regex():
    """this function will compile all needed regex patterns for the conversion
    returns a tuple of dictionaries for the meraki/__init__.py file and all files under the api directory
    """
    patternInitFile = {}
    patternAPIFile = {}

    # all patterns for meraki/__init__.py
    patternInitFile[re.compile("from \.legacy import \*\n")] = ""
    patternInitFile[re.compile("from \.config import")] = "from ..config import"
    patternInitFile[
        re.compile("class DashboardAPI\(object\):")
    ] = "class AsyncDashboardAPI:"
    patternInitFile[re.compile("(from \.api\..*import )(.*)")] = r"\1Async\2"
    patternInitFile[
        re.compile("self\._session = RestSession\(")
    ] = "self._session = AsyncRestSession("
    patternInitFile[re.compile("(self\..*) = (.*\(self\._session\))")] = r"\1 = Async\2"
    patternInitFile[
        re.compile("\n$")
    ] = """

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self._session.close()
"""

    # patterns for meraki/api/*.py files
    patternAPIFile[
        re.compile("super\([a-zA-Z0-9_]+, self\)\.__init__\(\)")
    ] = "super().__init__()"
    patternAPIFile[re.compile("class ([a-zA-Z0-9]+)\(object\):")] = r"class Async\1:"
    patternAPIFile[re.compile("def ([^_].*)")] = r"async def \1"
    patternAPIFile[
        re.compile("return self\._session\.")
    ] = "return await self._session."

    return patternInitFile, patternAPIFile


def convert_file(path, filename, patternDict):
    content = read_file(os.path.join(path, filename))

    _logger.info("Starting Conversion")

    for p, v in patternDict.items():
        content, count = p.subn(v, content)
        _logger.debug(f"Applied pattern {p.pattern} {count} times")

    _logger.info("Finished Conversion")

    aioFile = get_aio_path(path, filename)
    with open(aioFile, "w") as f:
        f.write(content)
    _logger.info(f"File {aioFile} saved")


def main():
    # make sure that the aio/api directory does exist
    if not os.path.exists("meraki/aio"):
        os.makedirs("meraki/aio")
    if not os.path.exists("meraki/aio/api"):
        os.makedirs("meraki/aio/api")

    patternInitFile, patternAPIFile = compile_regex()
    convert_file("meraki", "__init__.py", patternInitFile)
    apifolder = "meraki/api"

    for file in os.listdir("meraki/api"):
        if os.path.isfile(os.path.join(apifolder, file)) and file[-3:] == ".py":
            convert_file(apifolder, file, patternAPIFile)


if __name__ == "__main__":
    start_time = datetime.now()
    _logger = create_logger(log_file_prefix=__file__[:-3], print_console=True)
    main()
    end_time = datetime.now()
    _logger.info(f"\nScript complete, total runtime {end_time - start_time}")



class AsyncDashboardBrandingPolicies:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getOrganizationBrandingPolicies(self, organizationId: str):
        """
        **List the branding policies of an organization**
        https://api.meraki.com/api_docs#list-the-branding-policies-of-an-organization
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['Dashboard branding policies'],
            'operation': 'getOrganizationBrandingPolicies',
        }
        resource = f'/organizations/{organizationId}/brandingPolicies'

        return await self._session.get(metadata, resource)

    async def createOrganizationBrandingPolicy(self, organizationId: str, name: str, enabled: bool, adminSettings: dict, **kwargs):
        """
        **Add a new branding policy to an organization**
        https://api.meraki.com/api_docs#add-a-new-branding-policy-to-an-organization
        
        - organizationId (string)
        - name (string): Name of the Dashboard branding policy.
        - enabled (boolean): Boolean indicating whether this policy is enabled.
        - adminSettings (object): Settings for describing which kinds of admins this policy applies to.
        - helpSettings (object):     Settings for describing the modifications to various Help page features. Each property in this object accepts one of
    'default or inherit' (do not modify functionality), 'hide' (remove the section from Dashboard), or 'show' (always show
    the section on Dashboard). Some properties in this object also accept custom HTML used to replace the section on
    Dashboard; see the documentation for each property to see the allowed values.
 Each property defaults to 'default or inherit' when not provided.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['Dashboard branding policies'],
            'operation': 'createOrganizationBrandingPolicy',
        }
        resource = f'/organizations/{organizationId}/brandingPolicies'

        body_params = ['name', 'enabled', 'adminSettings', 'helpSettings']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.post(metadata, resource, payload)

    async def getOrganizationBrandingPoliciesPriorities(self, organizationId: str):
        """
        **Return the branding policy IDs of an organization in priority order. IDs are ordered in ascending order of priority (IDs later in the array have higher priority).**
        https://api.meraki.com/api_docs#return-the-branding-policy-ids-of-an-organization-in-priority-order
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['Dashboard branding policies'],
            'operation': 'getOrganizationBrandingPoliciesPriorities',
        }
        resource = f'/organizations/{organizationId}/brandingPolicies/priorities'

        return await self._session.get(metadata, resource)

    async def updateOrganizationBrandingPoliciesPriorities(self, organizationId: str, brandingPolicyIds: list):
        """
        **Update the priority ordering of an organization's branding policies.**
        https://api.meraki.com/api_docs#update-the-priority-ordering-of-an-organizations-branding-policies
        
        - organizationId (string)
        - brandingPolicyIds (array): A list of branding policy IDs arranged in ascending priority order (IDs later in the array have higher priority).
        """

        kwargs = locals()

        metadata = {
            'tags': ['Dashboard branding policies'],
            'operation': 'updateOrganizationBrandingPoliciesPriorities',
        }
        resource = f'/organizations/{organizationId}/brandingPolicies/priorities'

        body_params = ['brandingPolicyIds']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)

    async def getOrganizationBrandingPolicy(self, organizationId: str, brandingPolicyId: str):
        """
        **Return a branding policy**
        https://api.meraki.com/api_docs#return-a-branding-policy
        
        - organizationId (string)
        - brandingPolicyId (string)
        """

        metadata = {
            'tags': ['Dashboard branding policies'],
            'operation': 'getOrganizationBrandingPolicy',
        }
        resource = f'/organizations/{organizationId}/brandingPolicies/{brandingPolicyId}'

        return await self._session.get(metadata, resource)

    async def updateOrganizationBrandingPolicy(self, organizationId: str, brandingPolicyId: str, **kwargs):
        """
        **Update a branding policy**
        https://api.meraki.com/api_docs#update-a-branding-policy
        
        - organizationId (string)
        - brandingPolicyId (string)
        - name (string): Name of the Dashboard branding policy.
        - enabled (boolean): Boolean indicating whether this policy is enabled.
        - adminSettings (object): Settings for describing which kinds of admins this policy applies to.
        - helpSettings (object):     Settings for describing the modifications to various Help page features. Each property in this object accepts one of
    'default or inherit' (do not modify functionality), 'hide' (remove the section from Dashboard), or 'show' (always show
    the section on Dashboard). Some properties in this object also accept custom HTML used to replace the section on
    Dashboard; see the documentation for each property to see the allowed values.

        """

        kwargs.update(locals())

        metadata = {
            'tags': ['Dashboard branding policies'],
            'operation': 'updateOrganizationBrandingPolicy',
        }
        resource = f'/organizations/{organizationId}/brandingPolicies/{brandingPolicyId}'

        body_params = ['name', 'enabled', 'adminSettings', 'helpSettings']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)

    async def deleteOrganizationBrandingPolicy(self, organizationId: str, brandingPolicyId: str):
        """
        **Delete a branding policy**
        https://api.meraki.com/api_docs#delete-a-branding-policy
        
        - organizationId (string)
        - brandingPolicyId (string)
        """

        metadata = {
            'tags': ['Dashboard branding policies'],
            'operation': 'deleteOrganizationBrandingPolicy',
        }
        resource = f'/organizations/{organizationId}/brandingPolicies/{brandingPolicyId}'

        return await self._session.delete(metadata, resource)




from datetime import datetime
import logging
import os

from .legacy import *
from .rest_session import *
from .api.api_usage import APIUsage
from .api.action_batches import ActionBatches
from .api.admins import Admins
from .api.alert_settings import AlertSettings
from .api.bluetooth_clients import BluetoothClients
from .api.bluetooth_settings import BluetoothSettings
from .api.camera_quality_retention_profiles import CameraQualityRetentionProfiles
from .api.cameras import Cameras
from .api.change_log import ChangeLog
from .api.clients import Clients
from .api.config_templates import ConfigTemplates
from .api.connectivity_monitoring_destinations import ConnectivityMonitoringDestinations
from .api.content_filtering_categories import ContentFilteringCategories
from .api.content_filtering_rules import ContentFilteringRules
from .api.dashboard_branding_policies import DashboardBrandingPolicies
from .api.devices import Devices
from .api.events import Events
from .api.firewalled_services import FirewalledServices
from .api.floorplans import Floorplans
from .api.group_policies import GroupPolicies
from .api.http_servers import HTTPServers
from .api.intrusion_settings import IntrusionSettings
from .api.licenses import Licenses
from .api.link_aggregations import LinkAggregations
from .api.mg_dhcp_settings import MGDHCPSettings
from .api.mg_lan_settings import MGLANSettings
from .api.mg_connectivity_monitoring_destinations import MGConnectivityMonitoringDestinations
from .api.mg_port_forwarding_rules import MGPortForwardingRules
from .api.mg_subnet_pool_settings import MGSubnetPoolSettings
from .api.mg_uplink_settings import MGUplinkSettings
from .api.mr_l3_firewall import MRL3Firewall
from .api.mv_sense import MVSense
from .api.mx_1_1_nat_rules import MX11NATRules
from .api.mx_1_many_nat_rules import MX1ManyNATRules
from .api.mx_l3_firewall import MXL3Firewall
from .api.mx_l7_application_categories import MXL7ApplicationCategories
from .api.mx_l7_firewall import MXL7Firewall
from .api.mx_vlan_ports import MXVLANPorts
from .api.mx_vpn_firewall import MXVPNFirewall
from .api.mx_cellular_firewall import MXCellularFirewall
from .api.mx_inbound_firewall import MXInboundFirewall
from .api.mx_port_forwarding_rules import MXPortForwardingRules
from .api.mx_static_routes import MXStaticRoutes
from .api.mx_warm_spare_settings import MXWarmSpareSettings
from .api.malware_settings import MalwareSettings
from .api.management_interface_settings import ManagementInterfaceSettings
from .api.meraki_auth_users import MerakiAuthUsers
from .api.monitored_media_servers import MonitoredMediaServers
from .api.named_tag_scope import NamedTagScope
from .api.netflow_settings import NetFlowSettings
from .api.networks import Networks
from .api.openapi_spec import OpenAPISpec
from .api.organizations import Organizations
from .api.pii import PII
from .api.radio_settings import RadioSettings
from .api.saml_roles import SAMLRoles
from .api.sm import SM
from .api.snmp_settings import SNMPSettings
from .api.ssids import SSIDs
from .api.security_events import SecurityEvents
from .api.splash_login_attempts import SplashLoginAttempts
from .api.splash_settings import SplashSettings
from .api.switch_acls import SwitchACLs
from .api.switch_port_schedules import SwitchPortSchedules
from .api.switch_ports import SwitchPorts
from .api.switch_profiles import SwitchProfiles
from .api.switch_settings import SwitchSettings
from .api.switch_stacks import SwitchStacks
from .api.syslog_servers import SyslogServers
from .api.traffic_analysis_settings import TrafficAnalysisSettings
from .api.traffic_shaping import TrafficShaping
from .api.uplink_settings import UplinkSettings
from .api.vlans import VLANs
from .api.webhook_logs import WebhookLogs
from .api.wireless_health import WirelessHealth
from .api.wireless_settings import WirelessSettings
from .config import (
    API_KEY_ENVIRONMENT_VARIABLE, DEFAULT_BASE_URL, SINGLE_REQUEST_TIMEOUT, CERTIFICATE_PATH, WAIT_ON_RATE_LIMIT,
    MAXIMUM_RETRIES, OUTPUT_LOG, LOG_PATH, LOG_FILE_PREFIX, PRINT_TO_CONSOLE, SIMULATE_API_CALLS
)

__version__ = '0.90.1'

class DashboardAPI(object):
    """
    **Creates a persistent Meraki dashboard API session**

    - api_key (string): API key generated in dashboard; can also be set as an environment variable MERAKI_DASHBOARD_API_KEY
    - base_url (string): preceding all endpoint resources
    - single_request_timeout (integer): maximum number of seconds for each API call
    - certificate_path (string): path for TLS/SSL certificate verification if behind local proxy
    - wait_on_rate_limit (boolean): retry if 429 rate limit error encountered?
    - maximum_retries (integer): retry up to this many times when encountering 429s or other server-side errors
    - output_log (boolean): create an output log file?
    - log_path (string): path to output log; by default, working directory of script if not specified
    - log_file_prefix (string): log file name appended with date and timestamp
    - print_console (boolean): print logging output to console?
    - simulate (boolean): simulate POST/PUT/DELETE calls to prevent changes?
    """

    def __init__(self, api_key=None, base_url=DEFAULT_BASE_URL, single_request_timeout=SINGLE_REQUEST_TIMEOUT,
                 certificate_path=CERTIFICATE_PATH, wait_on_rate_limit=WAIT_ON_RATE_LIMIT,
                 maximum_retries=MAXIMUM_RETRIES, output_log=OUTPUT_LOG, log_path=LOG_PATH,
                 log_file_prefix=LOG_FILE_PREFIX, print_console=PRINT_TO_CONSOLE, simulate=SIMULATE_API_CALLS):
        # Check API key
        api_key = api_key or os.environ.get(API_KEY_ENVIRONMENT_VARIABLE)
        if not api_key:
            raise APIKeyError()

        # Configure logging
        self._logger = logging.getLogger(__name__)
        if log_path and log_path[-1] != '/':
            log_path += '/'
        self._log_file = f'{log_path}{log_file_prefix}_log__{datetime.now():%Y-%m-%d_%H-%M-%S}.log'
        if output_log:
            logging.basicConfig(
                filename=self._log_file,
                level=logging.DEBUG,
                format='%(asctime)s %(name)12s: %(levelname)8s > %(message)s',
                datefmt='%Y-%m-%d %H:%M:%S')
            if print_console:
                console = logging.StreamHandler()
                console.setLevel(logging.INFO)
                formatter = logging.Formatter('%(name)12s: %(levelname)8s > %(message)s')
                console.setFormatter(formatter)
                logging.getLogger('').addHandler(console)
        elif print_console:
            logging.basicConfig(
                level=logging.DEBUG,
                format='%(asctime)s %(name)12s: %(levelname)8s > %(message)s',
                datefmt='%Y-%m-%d %H:%M:%S')

        # Creates the API session
        self._session = RestSession(
            logger=self._logger,
            api_key=api_key,
            base_url=base_url,
            single_request_timeout=single_request_timeout,
            certificate_path=certificate_path,
            wait_on_rate_limit=wait_on_rate_limit,
            maximum_retries=maximum_retries,
            simulate=simulate,
        )

        # API endpoints by section
        self.api_usage = APIUsage(self._session)
        self.action_batches = ActionBatches(self._session)
        self.admins = Admins(self._session)
        self.alert_settings = AlertSettings(self._session)
        self.bluetooth_clients = BluetoothClients(self._session)
        self.bluetooth_settings = BluetoothSettings(self._session)
        self.camera_quality_retention_profiles = CameraQualityRetentionProfiles(self._session)
        self.cameras = Cameras(self._session)
        self.change_log = ChangeLog(self._session)
        self.clients = Clients(self._session)
        self.config_templates = ConfigTemplates(self._session)
        self.connectivity_monitoring_destinations = ConnectivityMonitoringDestinations(self._session)
        self.content_filtering_categories = ContentFilteringCategories(self._session)
        self.content_filtering_rules = ContentFilteringRules(self._session)
        self.dashboard_branding_policies = DashboardBrandingPolicies(self._session)
        self.devices = Devices(self._session)
        self.events = Events(self._session)
        self.firewalled_services = FirewalledServices(self._session)
        self.floorplans = Floorplans(self._session)
        self.group_policies = GroupPolicies(self._session)
        self.http_servers = HTTPServers(self._session)
        self.intrusion_settings = IntrusionSettings(self._session)
        self.licenses = Licenses(self._session)
        self.link_aggregations = LinkAggregations(self._session)
        self.mg_dhcp_settings = MGDHCPSettings(self._session)
        self.mg_lan_settings = MGLANSettings(self._session)
        self.mg_connectivity_monitoring_destinations = MGConnectivityMonitoringDestinations(self._session)
        self.mg_port_forwarding_rules = MGPortForwardingRules(self._session)
        self.mg_subnet_pool_settings = MGSubnetPoolSettings(self._session)
        self.mg_uplink_settings = MGUplinkSettings(self._session)
        self.mr_l3_firewall = MRL3Firewall(self._session)
        self.mv_sense = MVSense(self._session)
        self.mx_1_1_nat_rules = MX11NATRules(self._session)
        self.mx_1_many_nat_rules = MX1ManyNATRules(self._session)
        self.mx_l3_firewall = MXL3Firewall(self._session)
        self.mx_l7_application_categories = MXL7ApplicationCategories(self._session)
        self.mx_l7_firewall = MXL7Firewall(self._session)
        self.mx_vlan_ports = MXVLANPorts(self._session)
        self.mx_vpn_firewall = MXVPNFirewall(self._session)
        self.mx_cellular_firewall = MXCellularFirewall(self._session)
        self.mx_inbound_firewall = MXInboundFirewall(self._session)
        self.mx_port_forwarding_rules = MXPortForwardingRules(self._session)
        self.mx_static_routes = MXStaticRoutes(self._session)
        self.mx_warm_spare_settings = MXWarmSpareSettings(self._session)
        self.malware_settings = MalwareSettings(self._session)
        self.management_interface_settings = ManagementInterfaceSettings(self._session)
        self.meraki_auth_users = MerakiAuthUsers(self._session)
        self.monitored_media_servers = MonitoredMediaServers(self._session)
        self.named_tag_scope = NamedTagScope(self._session)
        self.netflow_settings = NetFlowSettings(self._session)
        self.networks = Networks(self._session)
        self.openapi_spec = OpenAPISpec(self._session)
        self.organizations = Organizations(self._session)
        self.pii = PII(self._session)
        self.radio_settings = RadioSettings(self._session)
        self.saml_roles = SAMLRoles(self._session)
        self.sm = SM(self._session)
        self.snmp_settings = SNMPSettings(self._session)
        self.ssids = SSIDs(self._session)
        self.security_events = SecurityEvents(self._session)
        self.splash_login_attempts = SplashLoginAttempts(self._session)
        self.splash_settings = SplashSettings(self._session)
        self.switch_acls = SwitchACLs(self._session)
        self.switch_port_schedules = SwitchPortSchedules(self._session)
        self.switch_ports = SwitchPorts(self._session)
        self.switch_profiles = SwitchProfiles(self._session)
        self.switch_settings = SwitchSettings(self._session)
        self.switch_stacks = SwitchStacks(self._session)
        self.syslog_servers = SyslogServers(self._session)
        self.traffic_analysis_settings = TrafficAnalysisSettings(self._session)
        self.traffic_shaping = TrafficShaping(self._session)
        self.uplink_settings = UplinkSettings(self._session)
        self.vlans = VLANs(self._session)
        self.webhook_logs = WebhookLogs(self._session)
        self.wireless_health = WirelessHealth(self._session)
        self.wireless_settings = WirelessSettings(self._session)



class AsyncSwitchProfiles:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getOrganizationConfigTemplateSwitchProfiles(self, organizationId: str, configTemplateId: str):
        """
        **List the switch profiles for your switch template configuration**
        https://api.meraki.com/api_docs#list-the-switch-profiles-for-your-switch-template-configuration
        
        - organizationId (string)
        - configTemplateId (string)
        """

        metadata = {
            'tags': ['Switch profiles'],
            'operation': 'getOrganizationConfigTemplateSwitchProfiles',
        }
        resource = f'/organizations/{organizationId}/configTemplates/{configTemplateId}/switchProfiles'

        return await self._session.get(metadata, resource)




class MX1ManyNATRules(object):
    def __init__(self, session):
        super(MX1ManyNATRules, self).__init__()
        self._session = session
    
    def getNetworkOneToManyNatRules(self, networkId: str):
        """
        **Return the 1:Many NAT mapping rules for an MX network**
        https://api.meraki.com/api_docs#return-the-1many-nat-mapping-rules-for-an-mx-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['MX 1:Many NAT rules'],
            'operation': 'getNetworkOneToManyNatRules',
        }
        resource = f'/networks/{networkId}/oneToManyNatRules'

        return self._session.get(metadata, resource)

    def updateNetworkOneToManyNatRules(self, networkId: str, rules: list):
        """
        **Set the 1:Many NAT mapping rules for an MX network**
        https://api.meraki.com/api_docs#set-the-1many-nat-mapping-rules-for-an-mx-network
        
        - networkId (string)
        - rules (array): An array of 1:Many nat rules
        """

        kwargs = locals()

        metadata = {
            'tags': ['MX 1:Many NAT rules'],
            'operation': 'updateNetworkOneToManyNatRules',
        }
        resource = f'/networks/{networkId}/oneToManyNatRules'

        body_params = ['rules']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)




class AsyncUplinkSettings:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getNetworkUplinkSettings(self, networkId: str):
        """
        **Returns the uplink settings for your MX network.**
        https://api.meraki.com/api_docs#returns-the-uplink-settings-for-your-mx-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['Uplink settings'],
            'operation': 'getNetworkUplinkSettings',
        }
        resource = f'/networks/{networkId}/uplinkSettings'

        return await self._session.get(metadata, resource)

    async def updateNetworkUplinkSettings(self, networkId: str, **kwargs):
        """
        **Updates the uplink settings for your MX network.**
        https://api.meraki.com/api_docs#updates-the-uplink-settings-for-your-mx-network
        
        - networkId (string)
        - bandwidthLimits (object): A mapping of uplinks to their bandwidth settings (be sure to check which uplinks are supported for your network)
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['Uplink settings'],
            'operation': 'updateNetworkUplinkSettings',
        }
        resource = f'/networks/{networkId}/uplinkSettings'

        body_params = ['bandwidthLimits']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)




class Organizations(object):
    def __init__(self, session):
        super(Organizations, self).__init__()
        self._session = session
    
    def getOrganizations(self):
        """
        **List the organizations that the user has privileges on**
        https://api.meraki.com/api_docs#list-the-organizations-that-the-user-has-privileges-on
        
        """

        metadata = {
            'tags': ['Organizations'],
            'operation': 'getOrganizations',
        }
        resource = f'/organizations'

        return self._session.get(metadata, resource)

    def createOrganization(self, name: str):
        """
        **Create a new organization**
        https://api.meraki.com/api_docs#create-a-new-organization
        
        - name (string): The name of the organization
        """

        kwargs = locals()

        metadata = {
            'tags': ['Organizations'],
            'operation': 'createOrganization',
        }
        resource = f'/organizations'

        body_params = ['name']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.post(metadata, resource, payload)

    def getOrganization(self, organizationId: str):
        """
        **Return an organization**
        https://api.meraki.com/api_docs#return-an-organization
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['Organizations'],
            'operation': 'getOrganization',
        }
        resource = f'/organizations/{organizationId}'

        return self._session.get(metadata, resource)

    def updateOrganization(self, organizationId: str, **kwargs):
        """
        **Update an organization**
        https://api.meraki.com/api_docs#update-an-organization
        
        - organizationId (string)
        - name (string): The name of the organization
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['Organizations'],
            'operation': 'updateOrganization',
        }
        resource = f'/organizations/{organizationId}'

        body_params = ['name']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)

    def deleteOrganization(self, organizationId: str):
        """
        **Delete an organization**
        https://api.meraki.com/api_docs#delete-an-organization
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['Organizations'],
            'operation': 'deleteOrganization',
        }
        resource = f'/organizations/{organizationId}'

        return self._session.delete(metadata, resource)

    def claimOrganization(self, organizationId: str, **kwargs):
        """
        **Claim a list of devices, licenses, and/or orders into an organization. When claiming by order, all devices and licenses in the order will be claimed; licenses will be added to the organization and devices will be placed in the organization's inventory.**
        https://api.meraki.com/api_docs#claim-a-list-of-devices-licenses-and/or-orders-into-an-organization
        
        - organizationId (string)
        - orders (array): The numbers of the orders that should be claimed
        - serials (array): The serials of the devices that should be claimed
        - licenses (array): The licenses that should be claimed
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['Organizations'],
            'operation': 'claimOrganization',
        }
        resource = f'/organizations/{organizationId}/claim'

        body_params = ['orders', 'serials', 'licenses']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.post(metadata, resource, payload)

    def cloneOrganization(self, organizationId: str, name: str):
        """
        **Create a new organization by cloning the addressed organization**
        https://api.meraki.com/api_docs#create-a-new-organization-by-cloning-the-addressed-organization
        
        - organizationId (string)
        - name (string): The name of the new organization
        """

        kwargs = locals()

        metadata = {
            'tags': ['Organizations'],
            'operation': 'cloneOrganization',
        }
        resource = f'/organizations/{organizationId}/clone'

        body_params = ['name']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.post(metadata, resource, payload)

    def getOrganizationDeviceStatuses(self, organizationId: str):
        """
        **List the status of every Meraki device in the organization**
        https://api.meraki.com/api_docs#list-the-status-of-every-meraki-device-in-the-organization
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['Organizations'],
            'operation': 'getOrganizationDeviceStatuses',
        }
        resource = f'/organizations/{organizationId}/deviceStatuses'

        return self._session.get(metadata, resource)

    def getOrganizationInventory(self, organizationId: str, **kwargs):
        """
        **Return the inventory for an organization**
        https://api.meraki.com/api_docs#return-the-inventory-for-an-organization
        
        - organizationId (string)
        - includeLicenseInfo (boolean): When this parameter is true, each entity in the response will include the license expiration date of the device (if any). Only applies to organizations that support per-device licensing. Defaults to false.
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['Organizations'],
            'operation': 'getOrganizationInventory',
        }
        resource = f'/organizations/{organizationId}/inventory'

        query_params = ['includeLicenseInfo']
        params = {k: v for (k, v) in kwargs.items() if k in query_params}

        return self._session.get(metadata, resource, params)

    def getOrganizationLicenseState(self, organizationId: str):
        """
        **Return an overview of the license state for an organization**
        https://api.meraki.com/api_docs#return-an-overview-of-the-license-state-for-an-organization
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['Organizations'],
            'operation': 'getOrganizationLicenseState',
        }
        resource = f'/organizations/{organizationId}/licenseState'

        return self._session.get(metadata, resource)

    def getOrganizationThirdPartyVPNPeers(self, organizationId: str):
        """
        **Return the third party VPN peers for an organization**
        https://api.meraki.com/api_docs#return-the-third-party-vpn-peers-for-an-organization
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['Organizations'],
            'operation': 'getOrganizationThirdPartyVPNPeers',
        }
        resource = f'/organizations/{organizationId}/thirdPartyVPNPeers'

        return self._session.get(metadata, resource)

    def updateOrganizationThirdPartyVPNPeers(self, organizationId: str, peers: list):
        """
        **Update the third party VPN peers for an organization**
        https://api.meraki.com/api_docs#update-the-third-party-vpn-peers-for-an-organization
        
        - organizationId (string)
        - peers (array): The list of VPN peers
        """

        kwargs = locals()

        metadata = {
            'tags': ['Organizations'],
            'operation': 'updateOrganizationThirdPartyVPNPeers',
        }
        resource = f'/organizations/{organizationId}/thirdPartyVPNPeers'

        body_params = ['peers']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)

    def getOrganizationUplinksLossAndLatency(self, organizationId: str, **kwargs):
        """
        **Return the uplink loss and latency for every MX in the organization from at latest 2 minutes ago**
        https://api.meraki.com/api_docs#return-the-uplink-loss-and-latency-for-every-mx-in-the-organization-from-at-latest-2-minutes-ago
        
        - organizationId (string)
        - t0 (string): The beginning of the timespan for the data. The maximum lookback period is 365 days from today.
        - t1 (string): The end of the timespan for the data. t1 can be a maximum of 5 minutes after t0. The latest possible time that t1 can be is 2 minutes into the past.
        - timespan (number): The timespan for which the information will be fetched. If specifying timespan, do not specify parameters t0 and t1. The value must be in seconds and be less than or equal to 5 minutes. The default is 5 minutes.
        - uplink (string): Optional filter for a specific WAN uplink. Valid uplinks are wan1, wan2, cellular. Default will return all uplinks.
        - ip (string): Optional filter for a specific destination IP. Default will return all destination IPs.
        """

        kwargs.update(locals())

        if 'uplink' in kwargs:
            options = ['wan1', 'wan2', 'cellular']
            assert kwargs['uplink'] in options, f'''"uplink" cannot be "{kwargs['uplink']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['Organizations'],
            'operation': 'getOrganizationUplinksLossAndLatency',
        }
        resource = f'/organizations/{organizationId}/uplinksLossAndLatency'

        query_params = ['t0', 't1', 'timespan', 'uplink', 'ip']
        params = {k: v for (k, v) in kwargs.items() if k in query_params}

        return self._session.get(metadata, resource, params)




class ContentFilteringCategories(object):
    def __init__(self, session):
        super(ContentFilteringCategories, self).__init__()
        self._session = session
    
    def getNetworkContentFilteringCategories(self, networkId: str):
        """
        **List all available content filtering categories for an MX network**
        https://api.meraki.com/api_docs#list-all-available-content-filtering-categories-for-an-mx-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['Content filtering categories'],
            'operation': 'getNetworkContentFilteringCategories',
        }
        resource = f'/networks/{networkId}/contentFiltering/categories'

        return self._session.get(metadata, resource)




class AsyncMGLANSettings:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getDeviceCellularGatewaySettings(self, serial: str):
        """
        **Show the LAN Settings of a MG**
        https://api.meraki.com/api_docs#show-the-lan-settings-of-a-mg
        
        - serial (string)
        """

        metadata = {
            'tags': ['MG LAN settings'],
            'operation': 'getDeviceCellularGatewaySettings',
        }
        resource = f'/devices/{serial}/cellularGateway/settings'

        return await self._session.get(metadata, resource)

    async def updateDeviceCellularGatewaySettings(self, serial: str, **kwargs):
        """
        **Update the LAN Settings for a single MG.**
        https://api.meraki.com/api_docs#update-the-lan-settings-for-a-single-mg
        
        - serial (string)
        - reservedIpRanges (array): list of all reserved IP ranges for a single MG
        - fixedIpAssignments (array): list of all fixed IP assignments for a single MG
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['MG LAN settings'],
            'operation': 'updateDeviceCellularGatewaySettings',
        }
        resource = f'/devices/{serial}/cellularGateway/settings'

        body_params = ['reservedIpRanges', 'fixedIpAssignments']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)




class SAMLRoles(object):
    def __init__(self, session):
        super(SAMLRoles, self).__init__()
        self._session = session
    
    def getOrganizationSamlRoles(self, organizationId: str):
        """
        **List the SAML roles for this organization**
        https://api.meraki.com/api_docs#list-the-saml-roles-for-this-organization
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['SAML roles'],
            'operation': 'getOrganizationSamlRoles',
        }
        resource = f'/organizations/{organizationId}/samlRoles'

        return self._session.get(metadata, resource)

    def createOrganizationSamlRole(self, organizationId: str, **kwargs):
        """
        **Create a SAML role**
        https://api.meraki.com/api_docs#create-a-saml-role
        
        - organizationId (string)
        - role (string): The role of the SAML administrator
        - orgAccess (string): The privilege of the SAML administrator on the organization
        - tags (array): The list of tags that the SAML administrator has privleges on
        - networks (array): The list of networks that the SAML administrator has privileges on
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['SAML roles'],
            'operation': 'createOrganizationSamlRole',
        }
        resource = f'/organizations/{organizationId}/samlRoles'

        body_params = ['role', 'orgAccess', 'tags', 'networks']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.post(metadata, resource, payload)

    def getOrganizationSamlRole(self, organizationId: str, samlRoleId: str):
        """
        **Return a SAML role**
        https://api.meraki.com/api_docs#return-a-saml-role
        
        - organizationId (string)
        - samlRoleId (string)
        """

        metadata = {
            'tags': ['SAML roles'],
            'operation': 'getOrganizationSamlRole',
        }
        resource = f'/organizations/{organizationId}/samlRoles/{samlRoleId}'

        return self._session.get(metadata, resource)

    def updateOrganizationSamlRole(self, organizationId: str, samlRoleId: str, **kwargs):
        """
        **Update a SAML role**
        https://api.meraki.com/api_docs#update-a-saml-role
        
        - organizationId (string)
        - samlRoleId (string)
        - role (string): The role of the SAML administrator
        - orgAccess (string): The privilege of the SAML administrator on the organization
        - tags (array): The list of tags that the SAML administrator has privleges on
        - networks (array): The list of networks that the SAML administrator has privileges on
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['SAML roles'],
            'operation': 'updateOrganizationSamlRole',
        }
        resource = f'/organizations/{organizationId}/samlRoles/{samlRoleId}'

        body_params = ['role', 'orgAccess', 'tags', 'networks']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)

    def deleteOrganizationSamlRole(self, organizationId: str, samlRoleId: str):
        """
        **Remove a SAML role**
        https://api.meraki.com/api_docs#remove-a-saml-role
        
        - organizationId (string)
        - samlRoleId (string)
        """

        metadata = {
            'tags': ['SAML roles'],
            'operation': 'deleteOrganizationSamlRole',
        }
        resource = f'/organizations/{organizationId}/samlRoles/{samlRoleId}'

        return self._session.delete(metadata, resource)




class AsyncMalwareSettings:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getNetworkSecurityMalwareSettings(self, networkId: str):
        """
        **Returns all supported malware settings for an MX network**
        https://api.meraki.com/api_docs#returns-all-supported-malware-settings-for-an-mx-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['Malware settings'],
            'operation': 'getNetworkSecurityMalwareSettings',
        }
        resource = f'/networks/{networkId}/security/malwareSettings'

        return await self._session.get(metadata, resource)

    async def updateNetworkSecurityMalwareSettings(self, networkId: str, mode: str, **kwargs):
        """
        **Set the supported malware settings for an MX network**
        https://api.meraki.com/api_docs#set-the-supported-malware-settings-for-an-mx-network
        
        - networkId (string)
        - mode (string): Set mode to 'enabled' to enable malware prevention, otherwise 'disabled'
        - allowedUrls (array): The urls that should be permitted by the malware detection engine. If omitted, the current config will remain unchanged. This is available only if your network supports AMP whitelisting
        - allowedFiles (array): The sha256 digests of files that should be permitted by the malware detection engine. If omitted, the current config will remain unchanged. This is available only if your network supports AMP whitelisting
        """

        kwargs.update(locals())

        if 'mode' in kwargs:
            options = ['enabled', 'disabled']
            assert kwargs['mode'] in options, f'''"mode" cannot be "{kwargs['mode']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['Malware settings'],
            'operation': 'updateNetworkSecurityMalwareSettings',
        }
        resource = f'/networks/{networkId}/security/malwareSettings'

        body_params = ['mode', 'allowedUrls', 'allowedFiles']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)




class AsyncMGUplinkSettings:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getNetworkCellularGatewaySettingsUplink(self, networkId: str):
        """
        **Returns the uplink settings for your MG network.**
        https://api.meraki.com/api_docs#returns-the-uplink-settings-for-your-mg-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['MG uplink settings'],
            'operation': 'getNetworkCellularGatewaySettingsUplink',
        }
        resource = f'/networks/{networkId}/cellularGateway/settings/uplink'

        return await self._session.get(metadata, resource)

    async def updateNetworkCellularGatewaySettingsUplink(self, networkId: str, **kwargs):
        """
        **Updates the uplink settings for your MG network.**
        https://api.meraki.com/api_docs#updates-the-uplink-settings-for-your-mg-network
        
        - networkId (string)
        - bandwidthLimits (object): The bandwidth settings for the 'cellular' uplink
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['MG uplink settings'],
            'operation': 'updateNetworkCellularGatewaySettingsUplink',
        }
        resource = f'/networks/{networkId}/cellularGateway/settings/uplink'

        body_params = ['bandwidthLimits']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)




class AsyncSNMPSettings:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getNetworkSnmpSettings(self, networkId: str):
        """
        **Return the SNMP settings for a network**
        https://api.meraki.com/api_docs#return-the-snmp-settings-for-a-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['SNMP settings'],
            'operation': 'getNetworkSnmpSettings',
        }
        resource = f'/networks/{networkId}/snmpSettings'

        return await self._session.get(metadata, resource)

    async def updateNetworkSnmpSettings(self, networkId: str, **kwargs):
        """
        **Update the SNMP settings for a network**
        https://api.meraki.com/api_docs#update-the-snmp-settings-for-a-network
        
        - networkId (string)
        - access (string): The type of SNMP access. Can be one of 'none' (disabled), 'community' (V1/V2c), or 'users' (V3).
        - communityString (string): The SNMP community string. Only relevant if 'access' is set to 'community'.
        - users (array): The list of SNMP users. Only relevant if 'access' is set to 'users'.
        """

        kwargs.update(locals())

        if 'access' in kwargs:
            options = ['none', 'community', 'users']
            assert kwargs['access'] in options, f'''"access" cannot be "{kwargs['access']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['SNMP settings'],
            'operation': 'updateNetworkSnmpSettings',
        }
        resource = f'/networks/{networkId}/snmpSettings'

        body_params = ['access', 'communityString', 'users']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)

    async def getOrganizationSnmp(self, organizationId: str):
        """
        **Return the SNMP settings for an organization**
        https://api.meraki.com/api_docs#return-the-snmp-settings-for-an-organization
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['SNMP settings'],
            'operation': 'getOrganizationSnmp',
        }
        resource = f'/organizations/{organizationId}/snmp'

        return await self._session.get(metadata, resource)

    async def updateOrganizationSnmp(self, organizationId: str, **kwargs):
        """
        **Update the SNMP settings for an organization**
        https://api.meraki.com/api_docs#update-the-snmp-settings-for-an-organization
        
        - organizationId (string)
        - v2cEnabled (boolean): Boolean indicating whether SNMP version 2c is enabled for the organization.
        - v3Enabled (boolean): Boolean indicating whether SNMP version 3 is enabled for the organization.
        - v3AuthMode (string): The SNMP version 3 authentication mode. Can be either 'MD5' or 'SHA'.
        - v3AuthPass (string): The SNMP version 3 authentication password. Must be at least 8 characters if specified.
        - v3PrivMode (string): The SNMP version 3 privacy mode. Can be either 'DES' or 'AES128'.
        - v3PrivPass (string): The SNMP version 3 privacy password. Must be at least 8 characters if specified.
        - peerIps (string): The IPs that are allowed to access the SNMP server. This list should be IPv4 addresses separated by semi-colons (ie. "1.2.3.4;2.3.4.5").
        """

        kwargs.update(locals())

        if 'v3AuthMode' in kwargs:
            options = ['MD5', 'SHA']
            assert kwargs['v3AuthMode'] in options, f'''"v3AuthMode" cannot be "{kwargs['v3AuthMode']}", & must be set to one of: {options}'''
        if 'v3PrivMode' in kwargs:
            options = ['DES', 'AES128']
            assert kwargs['v3PrivMode'] in options, f'''"v3PrivMode" cannot be "{kwargs['v3PrivMode']}", & must be set to one of: {options}'''

        metadata = {
            'tags': ['SNMP settings'],
            'operation': 'updateOrganizationSnmp',
        }
        resource = f'/organizations/{organizationId}/snmp'

        body_params = ['v2cEnabled', 'v3Enabled', 'v3AuthMode', 'v3AuthPass', 'v3PrivMode', 'v3PrivPass', 'peerIps']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)




class MGConnectivityMonitoringDestinations(object):
    def __init__(self, session):
        super(MGConnectivityMonitoringDestinations, self).__init__()
        self._session = session
    
    def getNetworkCellularGatewaySettingsConnectivityMonitoringDestinations(self, networkId: str):
        """
        **Return the connectivity testing destinations for an MG network**
        https://api.meraki.com/api_docs#return-the-connectivity-testing-destinations-for-an-mg-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['MG connectivity monitoring destinations'],
            'operation': 'getNetworkCellularGatewaySettingsConnectivityMonitoringDestinations',
        }
        resource = f'/networks/{networkId}/cellularGateway/settings/connectivityMonitoringDestinations'

        return self._session.get(metadata, resource)

    def updateNetworkCellularGatewaySettingsConnectivityMonitoringDestinations(self, networkId: str, **kwargs):
        """
        **Update the connectivity testing destinations for an MG network**
        https://api.meraki.com/api_docs#update-the-connectivity-testing-destinations-for-an-mg-network
        
        - networkId (string)
        - destinations (array): The list of connectivity monitoring destinations
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['MG connectivity monitoring destinations'],
            'operation': 'updateNetworkCellularGatewaySettingsConnectivityMonitoringDestinations',
        }
        resource = f'/networks/{networkId}/cellularGateway/settings/connectivityMonitoringDestinations'

        body_params = ['destinations']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)




class MerakiAuthUsers(object):
    def __init__(self, session):
        super(MerakiAuthUsers, self).__init__()
        self._session = session
    
    def getNetworkMerakiAuthUsers(self, networkId: str):
        """
        **List the splash or RADIUS users configured under Meraki Authentication for a network**
        https://api.meraki.com/api_docs#list-the-splash-or-radius-users-configured-under-meraki-authentication-for-a-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['Meraki auth users'],
            'operation': 'getNetworkMerakiAuthUsers',
        }
        resource = f'/networks/{networkId}/merakiAuthUsers'

        return self._session.get(metadata, resource)

    def getNetworkMerakiAuthUser(self, networkId: str, merakiAuthUserId: str):
        """
        **Return the Meraki Auth splash or RADIUS user**
        https://api.meraki.com/api_docs#return-the-meraki-auth-splash-or-radius-user
        
        - networkId (string)
        - merakiAuthUserId (string)
        """

        metadata = {
            'tags': ['Meraki auth users'],
            'operation': 'getNetworkMerakiAuthUser',
        }
        resource = f'/networks/{networkId}/merakiAuthUsers/{merakiAuthUserId}'

        return self._session.get(metadata, resource)




class ConnectivityMonitoringDestinations(object):
    def __init__(self, session):
        super(ConnectivityMonitoringDestinations, self).__init__()
        self._session = session
    
    def getNetworkConnectivityMonitoringDestinations(self, networkId: str):
        """
        **Return the connectivity testing destinations for an MX network**
        https://api.meraki.com/api_docs#return-the-connectivity-testing-destinations-for-an-mx-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['Connectivity monitoring destinations'],
            'operation': 'getNetworkConnectivityMonitoringDestinations',
        }
        resource = f'/networks/{networkId}/connectivityMonitoringDestinations'

        return self._session.get(metadata, resource)

    def updateNetworkConnectivityMonitoringDestinations(self, networkId: str, **kwargs):
        """
        **Update the connectivity testing destinations for an MX network**
        https://api.meraki.com/api_docs#update-the-connectivity-testing-destinations-for-an-mx-network
        
        - networkId (string)
        - destinations (array): The list of connectivity monitoring destinations
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['Connectivity monitoring destinations'],
            'operation': 'updateNetworkConnectivityMonitoringDestinations',
        }
        resource = f'/networks/{networkId}/connectivityMonitoringDestinations'

        body_params = ['destinations']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)




"""Setup script for meraki"""

import os.path
from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(HERE, 'README.md')) as fid:
    README = fid.read()

setup(
    name='meraki',
    version='0.90.2',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['requests'],
    keywords = ['meraki', 'dashboard', 'cisco'],
    description='Cisco Meraki Dashboard API library',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/meraki/dashboard-api-python',
    author='Cisco Meraki',
    author_email='api-feedback@meraki.net',
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
)



class ConfigTemplates(object):
    def __init__(self, session):
        super(ConfigTemplates, self).__init__()
        self._session = session
    
    def getOrganizationConfigTemplates(self, organizationId: str):
        """
        **List the configuration templates for this organization**
        https://api.meraki.com/api_docs#list-the-configuration-templates-for-this-organization
        
        - organizationId (string)
        """

        metadata = {
            'tags': ['Config templates'],
            'operation': 'getOrganizationConfigTemplates',
        }
        resource = f'/organizations/{organizationId}/configTemplates'

        return self._session.get(metadata, resource)

    def deleteOrganizationConfigTemplate(self, organizationId: str, configTemplateId: str):
        """
        **Remove a configuration template**
        https://api.meraki.com/api_docs#remove-a-configuration-template
        
        - organizationId (string)
        - configTemplateId (string)
        """

        metadata = {
            'tags': ['Config templates'],
            'operation': 'deleteOrganizationConfigTemplate',
        }
        resource = f'/organizations/{organizationId}/configTemplates/{configTemplateId}'

        return self._session.delete(metadata, resource)




class MXWarmSpareSettings(object):
    def __init__(self, session):
        super(MXWarmSpareSettings, self).__init__()
        self._session = session
    
    def swapNetworkWarmspare(self, networkId: str):
        """
        **Swap MX primary and warm spare appliances**
        https://api.meraki.com/api_docs#swap-mx-primary-and-warm-spare-appliances
        
        - networkId (string)
        """

        metadata = {
            'tags': ['MX warm spare settings'],
            'operation': 'swapNetworkWarmspare',
        }
        resource = f'/networks/{networkId}/swapWarmSpare'

        return self._session.post(metadata, resource)

    def getNetworkWarmSpareSettings(self, networkId: str):
        """
        **Return MX warm spare settings**
        https://api.meraki.com/api_docs#return-mx-warm-spare-settings
        
        - networkId (string)
        """

        metadata = {
            'tags': ['MX warm spare settings'],
            'operation': 'getNetworkWarmSpareSettings',
        }
        resource = f'/networks/{networkId}/warmSpareSettings'

        return self._session.get(metadata, resource)

    def updateNetworkWarmSpareSettings(self, networkId: str, enabled: bool, **kwargs):
        """
        **Update MX warm spare settings**
        https://api.meraki.com/api_docs#update-mx-warm-spare-settings
        
        - networkId (string)
        - enabled (boolean): Enable warm spare
        - spareSerial (string): Serial number of the warm spare appliance
        - uplinkMode (string): Uplink mode, either virtual or public
        - virtualIp1 (string): The WAN 1 shared IP
        - virtualIp2 (string): The WAN 2 shared IP
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['MX warm spare settings'],
            'operation': 'updateNetworkWarmSpareSettings',
        }
        resource = f'/networks/{networkId}/warmSpareSettings'

        body_params = ['enabled', 'spareSerial', 'uplinkMode', 'virtualIp1', 'virtualIp2']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return self._session.put(metadata, resource, payload)




class AsyncWirelessSettings:
    def __init__(self, session):
        super().__init__()
        self._session = session
    
    async def getNetworkWirelessSettings(self, networkId: str):
        """
        **Return the wireless settings for a network**
        https://api.meraki.com/api_docs#return-the-wireless-settings-for-a-network
        
        - networkId (string)
        """

        metadata = {
            'tags': ['Wireless settings'],
            'operation': 'getNetworkWirelessSettings',
        }
        resource = f'/networks/{networkId}/wireless/settings'

        return await self._session.get(metadata, resource)

    async def updateNetworkWirelessSettings(self, networkId: str, **kwargs):
        """
        **Update the wireless settings for a network**
        https://api.meraki.com/api_docs#update-the-wireless-settings-for-a-network
        
        - networkId (string)
        - meshingEnabled (boolean): Toggle for enabling or disabling meshing in a network
        - ipv6BridgeEnabled (boolean): Toggle for enabling or disabling IPv6 bridging in a network (Note: if enabled, SSIDs must also be configured to use bridge mode)
        - locationAnalyticsEnabled (boolean): Toggle for enabling or disabling location analytics for your network
        - ledLightsOn (boolean): Toggle for enabling or disabling LED lights on all APs in the network (making them run dark)
        """

        kwargs.update(locals())

        metadata = {
            'tags': ['Wireless settings'],
            'operation': 'updateNetworkWirelessSettings',
        }
        resource = f'/networks/{networkId}/wireless/settings'

        body_params = ['meshingEnabled', 'ipv6BridgeEnabled', 'locationAnalyticsEnabled', 'ledLightsOn']
        payload = {k: v for (k, v) in kwargs.items() if k in body_params}

        return await self._session.put(metadata, resource, payload)

